import dotenv from 'dotenv';
import { Bus, EventMessage, FisMessage } from "../interface/transport.interface";
import { ReceiverAdapter as ReceiverAdapterInterface, Transport, TransportEvent, TransportMessage, TransportService } from "../interface/connector.interface";
import { filter, Observable, Observer, Subscription, takeWhile } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'
import { Adapter } from './adapter.base';
import ConsoleLogger from '../utils/log.utils';
import { WrappedMessage } from '../utils/message.ordering';

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class ReceiverAdapter extends Adapter implements ReceiverAdapterInterface {
    private console!: ConsoleLogger

    constructor(adapterId: string, transportType: Transport, transportService: TransportService) {
        super()
        this.console = new ConsoleLogger(`${transportType}ReceiverAdapter`, ['adapter'])
        this.connector = transportService
        this.setAdapterProfile(adapterId, transportType)
        this.console.log({ message: `Just testing to see if receiverAdapter is instantiated properly ${this.adapterProfile, this.connector ? 'TransportService Instantiated' : 'Trnasport Service not instantiated'}` })
    }

    setAdapterProfile(id: string, transportType: Transport): void {
        this.adapterProfile = {
            id: id,
            transportType: transportType
        }
    }

    getMessageBus(bus: Bus): Observable<TransportEvent> {
        this.console.log({ message: `Connector getting message bus for this connector: ${this.adapterProfile.id}` })
        return new Observable((observable: Observer<TransportEvent>) => {
            if (bus == Bus.GeneralBus) {
                const subscription: Subscription = this.connector.subscribe().pipe(
                    filter((message: TransportEvent) => message.event === 'New Message'),
                    // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                    filter((message: TransportEvent) => (message.data as TransportMessage).target == this.adapterProfile.id),
                ).subscribe((message: TransportEvent) => {
                    this.console.log({ message: `Received ${(((message.data as TransportMessage).payload as WrappedMessage).payload as FisMessage).header.messageID} from ${((message.data as TransportMessage).target)}`, details: message })
                    observable.next(message);
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
            if (bus == Bus.ResponseMessageBus) {
                /// logic here
            }
            if (bus == Bus.NotificationMessageBus) {
                /// logic here
            }
            if (bus == Bus.ErrorMessageBus) {
                /// logic here
            }
        });
    }

}


